/* RTC Routinen fr eine DS1337 oder DS1307 von Maxim (I2C Bus)
*/

#include <avr/io.h>
#include <inttypes.h>
#include <stdio.h>

#include "rtc.h"
#include "i2cmaster.h"
#include "isdn-display.h"

// ***** rtc_init ***
void rtc_init()
{
	 //	Uhr starten (CH-Bit bei DS1307 in Byte 0)
	 i2c_start_wait(RTC_ADDR+I2C_WRITE);     // set device address and write mode
     i2c_write(0x00);
	 i2c_write(0x00);                        // write address = 0     
     i2c_stop();
}

// ***** rtc_initClock ****
void rtc_initClock(const char *setDate, const char *setTime)
{

	// Uhr stellen
	uint8_t Stunden10, Stunden, Minuten10, Minuten, Sekunden10, Sekunden;
	uint8_t Jahr10, Jahr, Monat10, Monat, Tag10, Tag;
	
	sscanf(setTime,"%c%c:%c%c:%c%c", &Stunden10, &Stunden, &Minuten10, &Minuten, &Sekunden10, &Sekunden);

	Stunden10-=48;
	Stunden-=48;
	Minuten10-=48;
	Minuten-=48;
	Sekunden10-=48;
	Sekunden-=48;

	SetTime(&Stunden10, &Stunden, &Minuten10, &Minuten, &Sekunden10, &Sekunden);
	
	sscanf(setDate,"%c%c.%c%c.%c%c",&Tag10, &Tag, &Monat10, &Monat, &Jahr10, &Jahr);

	Jahr10-=48;
	Jahr-=48;
	Monat10-=48;
	Monat-=48;
	Tag10-=48;
	Tag-=48;

	SetDate(&Jahr10, &Jahr, &Monat10, &Monat, &Tag10, &Tag);
}

// ***** SetTime ***
void SetTime(uint8_t volatile *Std10, uint8_t volatile *Std, uint8_t volatile *Min10, 
			 uint8_t volatile *Min, uint8_t volatile *Sek10, uint8_t volatile *Sek)
{	
	 uint8_t setvar;

	 i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x00);
	 setvar = (*Sek10<<4) + *Sek;
	 i2c_write(setvar); 
	 i2c_stop();

	 i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x01);
	 setvar = (*Min10<<4) + *Min;
	 i2c_write(setvar); 
	 i2c_stop();

	 i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x02);
	 setvar = (*Std10<<4) + *Std;
	 i2c_write(setvar); 
	 i2c_stop();

} 

// ***** SetDate ***
void SetDate(uint8_t volatile *Jahr10, uint8_t volatile *Jahr, uint8_t volatile *Monat10, 
			 uint8_t volatile *Monat, uint8_t volatile *Tag10, uint8_t volatile *Tag)
{	
	 uint8_t setvar;

	 i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x06);
	 setvar = (*Jahr10<<4) + *Jahr;
	 i2c_write(setvar); 
	 i2c_stop();

	 i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x05);
	 setvar = (*Monat10<<4) + *Monat;
	 i2c_write(setvar); 
	 i2c_stop();

	 i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x04);
	 setvar = (*Tag10<<4) + *Tag;
	 i2c_write(setvar); 
	 i2c_stop();

} 
// ***** ReadDate ****
void ReadDate(uint8_t volatile *Jahr, uint8_t volatile *Monat, uint8_t volatile *Tag)
{	uint8_t ret;

	// Lese Jahr 
     i2c_start_wait(RTC_ADDR+I2C_WRITE);

     i2c_write(0x06);
     i2c_rep_start(RTC_ADDR+I2C_READ);

     ret = i2c_readNak();                    // read one byte from DS1337
     i2c_stop();
	 *Jahr = (ret & 0x0F) + ((ret & 0xF0)>>4)*10;
	
	// Lese Monat 
     i2c_start_wait(RTC_ADDR+I2C_WRITE);

     i2c_write(0x05);
     i2c_rep_start(RTC_ADDR+I2C_READ);

     ret = i2c_readNak();                    // read one byte from DS1337
     i2c_stop();
	 *Monat = (ret & 0x0F) + ((ret & 0x10)>>4)*10;

	// Lese Tag 
     i2c_start_wait(RTC_ADDR+I2C_WRITE);

     i2c_write(0x04);
     i2c_rep_start(RTC_ADDR+I2C_READ);

     ret = i2c_readNak();                    // read one byte from DS1337
     i2c_stop();
	 *Tag = (ret & 0x0F) + ((ret & 0x30)>>4)*10;
	 return;

}
// ***** ReadTime ****
void ReadTime(uint8_t volatile *Std, uint8_t volatile *Min, uint8_t volatile *Sek)
{	
	uint8_t ret;

	// Lese Stunden 
     i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x02);
     i2c_rep_start(RTC_ADDR+I2C_READ);

     ret = i2c_readNak();                    // read one byte from DS1337
     i2c_stop();
	 *Std = (ret & 0x0F) + ((ret & 0x30)>>4)*10;

	// Lese Minuten 
     i2c_start_wait(RTC_ADDR+I2C_WRITE);
     i2c_write(0x01);
     i2c_rep_start(RTC_ADDR+I2C_READ);

     ret = i2c_readNak();                    // read one byte from DS1337
     i2c_stop();
	 *Min = (ret & 0x0F) + ((ret & 0x70)>>4)*10;
	
	// Lese Sekunden 
     i2c_start_wait(RTC_ADDR+I2C_WRITE);

     i2c_write(0x00);
     i2c_rep_start(RTC_ADDR+I2C_READ);

     ret = i2c_readNak();                    // read one byte from DS1337
     i2c_stop();
	 *Sek = (ret & 0x0F) + ((ret & 0x70)>>4) *10;

	 return;
}
